import os

import random
import string


def create_init_file(base_dir):
	open(os.path.join(base_dir, '__init__.py'), 'a').close()


def create_file(base_dir, name, other):
	with open(os.path.join(base_dir, name), 'w') as f:
		with open(other) as o:
			f.write(o.read())


def create_git_ignore(base_dir):
	path = os.path.join(base_dir, '.gitignore')
	if not os.path.exists(path):
		with open(path, 'w') as f:
			f.write(
				"*.py[co]\n*.egg*\nbuild\ncache\n.script\nconfig.json\n*.db\n*.log\n.project\n.pydevproject\n.settings\n*~\n\#*\#\n/.emacs.desktop\n/.emacs.desktop.lock\n.elc\nauto-save-list\ntramp\n.\#*\n*.swp\n*.swo\n.DS_Store\n._*\nThumbs.db\nDesktop.ini\n.idea\nnode_modules\n.env\nstatic")
		pass


def generate_key():
	return ''.join([random.SystemRandom().choice("{}{}{}".format(string.ascii_letters, string.digits, "!#$%&'()*+,-./:;<>?@[]^_{|}~")) for i in range(50)])



import os
from nodel.frameworks.common import *
from dotenv import set_key
from nodel.framework import modules

HERE = os.path.dirname(os.path.abspath(__file__))


def make_project(base_dir):
	# create environment file
	env_file = os.path.join(base_dir, '.env')
	if not os.path.exists(env_file):
		open(env_file, 'a').close()
		set_key(env_file, 'ADDRESS', 'localhost')
		set_key(env_file, 'PORT', '8000')
		set_key(env_file, 'DJANGO_SETTINGS_MODULE', 'core.config.dev')
	set_key(env_file, 'SECRET_KEY', generate_key())

	# create git ignore
	create_git_ignore(base_dir)

	# create modules
	path = os.path.join(base_dir, 'modules')
	if not os.path.exists(path):
		os.mkdir(path)
	path_base = os.path.join(path, 'base.txt')
	if not os.path.exists(path_base):
		open(path_base, 'a').close()
	modules.set_module(path_base, "django", "1.11.3")
	modules.set_module(path_base, "python-dotenv", "0.6.5")
	path_dev = os.path.join(path, 'dev.txt')
	if not os.path.exists(path_dev):
		with open(path_dev, 'w') as f:
			f.write('-r base.txt')
	path_prod = os.path.join(path, 'prod.txt')
	if not os.path.exists(path_prod):
		with open(path_prod, 'w') as f:
			f.write('-r base.txt')
	modules.set_module(path_prod, "gunicorn", "19.7.1")

	# create core django files
	path = os.path.join(base_dir, 'core')
	if not os.path.exists(path):
		os.mkdir(path)

	create_init_file(path)
	create_file(path, 'manage.py', os.path.join(HERE, 'manage.txt'))

	path = os.path.join(base_dir, 'core', 'core')
	if not os.path.exists(path):
		os.mkdir(path)
	create_init_file(path)
	create_file(path, 'urls.py', os.path.join(HERE, 'urls.txt'))
	create_file(path, 'wsgi.py', os.path.join(HERE, 'wsgi.txt'))

	path = os.path.join(base_dir, 'core', 'core', 'config')
	if not os.path.exists(path):
		os.mkdir(path)
	create_init_file(path)

	create_file(path, 'base.py', os.path.join(HERE, 'config_base.txt'))
	create_file(path, 'dev.py', os.path.join(HERE, 'config_dev.txt'))
	create_file(path, 'prod.py', os.path.join(HERE, 'config_prod.txt'))



from setuptools import find_packages, setup

setup(
	name='nodel',
	version='0.3.0',
	packages=find_packages(),
	url='https://github.com/ary4n/nodel',
	license='MIT',
	author='aryan',
	author_email='alikhaniaryan@live.com',
	description='django project manager',
	keywords='minimal django project manager',
	download_url="https://github.com/ary4n/nodel/archive/0.3.0.tar.gz",
	install_requires=[
		'python-dotenv',
	],
)


import os
import re
import codecs
from collections import OrderedDict

__escape_decoder = codecs.getdecoder('unicode_escape')
__posix_variable = re.compile('\$\{[^\}]*\}')


def decode_escaped(escaped):
	return __escape_decoder(escaped)[0]


def parse_module(module_path):
	with open(module_path) as f:
		for line in f:
			line = line.strip()

			if line.startswith('-r'):
				yield line, ''
			if not line or line.startswith('#') or line.startswith('-') or '==' not in line:
				continue

			k, v = line.split('==', 1)
			k, v = k.strip(), v.strip().encode('unicode-escape').decode('ascii')

			if len(v) > 0:
				quoted = v[0] == v[len(v) - 1] in ['"', "'"]

				if quoted:
					v = decode_escaped(v[1:-1])

			yield k, v


def module_values(module_path):
	values = OrderedDict(parse_module(module_path))
	return values


def set_module(module_path, module_to_set, version_to_set):
	module_to_set = str(module_to_set)
	version_to_set = str(version_to_set)
	if not os.path.exists(module_path):
		return None, module_to_set, version_to_set
	dotenv_as_dict = OrderedDict(parse_module(module_path))
	dotenv_as_dict[module_to_set] = version_to_set
	success = flatten_and_write(module_path, dotenv_as_dict)
	return success, module_to_set, version_to_set


def remove_module(module_path, module_to_unset):
	module_to_unset = str(module_to_unset)
	if not os.path.exists(module_path):
		return None, module_to_unset
	dotenv_as_dict = module_values(module_path)
	if module_to_unset in dotenv_as_dict:
		dotenv_as_dict.pop(module_to_unset, None)
	else:
		return None, module_to_unset
	success = flatten_and_write(module_path, dotenv_as_dict)
	return success, module_to_unset


def flatten_and_write(module_path, dotenv_as_dict):
	with open(module_path, "w") as f:
		for k, v in dotenv_as_dict.items():
			if k.startswith('-r'):
				f.write("%s\n" % k)
			else:
				f.write("%s==%s\n" % (k, v))
	return True
